@@ -10,9 +10,9 @@
 
 void sgemm(float *a, float *b, float *c, int n)
 {
-  dimension "fortran" a(n, n);
-  dimension "fortran" b(n, n);
-  dimension c(n, n);
+  dimension "fortran" a[n; n];
+  dimension "fortran" b[n; n];
+  dimension c[n; n];
 
   for (int i = 1; i <= n; ++i)
     for (int j = 1; j <= n; ++j)
@@ -20,9 +20,9 @@ void sgemm(float *a, float *b, float *c, int n)
       float tmp = 0;
 
       for (int k = 1; k <= n; ++k)
-        tmp += a(i,k)*b(k,j);
+        tmp += a[i;k]*b[k;j];
 
-      c(i-1,j-1) = tmp;
+      c[i-1;j-1] = tmp;
     }
 }
 
@@ -32,21 +32,19 @@ int DUDE;
 
 void print_mpole(complex double *mpole, int n)
 {
-  dimension "fortran" mpole(0:n, -n:n);
-
-  unsigned long int x = 0;
+  dimension "fortran" mpole[0:n; -n:n];
 
   for (int i = 0; i <= n; ++i)
     for (int j = -n; j <= n; ++j)
     {
-      printf("%f\n", mpole(i, j));
+      printf("%f\n", creal(mpole[i; j]));
     }
 }
 
 int main()
 {
   int n = 17;
-  dimension "fortran" mpole(0:n, -n:n);
+  dimension "fortran" mpole[0:n; -n:n];
 
   complex double *mpole = malloc(nitemsof(mpole)*sizeof(complex double));
   if (!mpole)
@@ -54,7 +52,7 @@ int main()
 
   for (long i = lboundof(mpole, 0); i <= uboundof(mpole, 0); ++i)
     for (long j = lboundof(mpole, 1); j <= uboundof(mpole, 1); ++j)
-      mpole(i,j) = i+j;
+      mpole[i;j] = i+j;
 
   print_mpole(mpole, n);
 
@@ -62,14 +60,14 @@ int main()
 
   // same code as above, now using CnD shorthand:
 
-  dimension "fortran" mpole2(0:n, -n:n);
+  dimension "fortran" mpole2[0:n; -n:n];
   CND_DECL_ALLOC_HEAP(complex double, mpole2);
   if (!mpole2)
     perror("alloc mpole2");
 
   CND_FOR_AXIS(i, mpole2, 0)
     CND_FOR_AXIS(j, mpole2, 1)
-      mpole2(i,j) = i*j;
+      mpole2[i; j] = i*j;
 
   print_mpole(mpole2, n);
 
