@@ -57,7 +57,7 @@ class CGenerator(object):
             # a name.
             return 'sizeof(%s)' % self.visit(n.expr)
         else:
-            return '%s%s' % (n.op, operand)
+            return '%s %s' % (n.op, operand)
 
     def visit_BinaryOp(self, n):
         lval_str = self._parenthesize_if(n.left,
@@ -101,9 +101,17 @@ class CGenerator(object):
         s += self._generate_type(n.type)
         return s
 
+    def visit_TypeOfDeclaration(self, n):
+        return "__typeof__(%s)" % self.visit(n.declaration)
+
+    def visit_TypeOfExpression(self, n):
+        return "__typeof__(%s)" % self.visit(n.expr)
+
     def visit_Cast(self, n):
-        s = '(' + self._generate_type(n.to_type) + ')'
-        return s + ' ' + self.visit(n.expr)
+        expr = self._parenthesize_unless_simple(n.expr)
+        return "(%s) %s" % (
+                self._generate_type(n.to_type),
+                expr)
 
     def visit_ExprList(self, n):
         visited_subexprs = []
@@ -114,6 +122,9 @@ class CGenerator(object):
                 visited_subexprs.append(self.visit(expr))
         return ', '.join(visited_subexprs)
 
+    def visit_TypeList(self, n):
+        return ', '.join(self.visit(ch) for ch in n.types)
+
     def visit_Enum(self, n):
         s = 'enum'
         if n.name: s += ' ' + n.name
