@@ -96,7 +96,8 @@ class CLexer(object):
         'FLOAT', 'FOR', 'GOTO', 'IF', 'INLINE', 'INT', 'LONG', 'REGISTER',
         'RESTRICT', 'RETURN', 'SHORT', 'SIGNED', 'SIZEOF', 'STATIC', 'STRUCT',
         'SWITCH', 'TYPEDEF', 'UNION', 'UNSIGNED', 'VOID',
-        'VOLATILE', 'WHILE', '__ATTRIBUTE__', '__ASM__', '__ASM'
+        'VOLATILE', 'WHILE', '__ATTRIBUTE__', '__ASM__', '__ASM', '__TYPEOF__',
+        '__REAL__', '__IMAG__', '__BUILTIN_TYPES_COMPATIBLE_P'
     )
 
     keyword_map = {}
@@ -205,7 +206,7 @@ class CLexer(object):
     # floating constants (K&R2: A.2.5.3)
     exponent_part = r"""([eE][-+]?[0-9]+)"""
     fractional_constant = r"""([0-9]*\.[0-9]+)|([0-9]+\.)"""
-    floating_constant = '(((('+fractional_constant+')'+exponent_part+'?)|([0-9]+'+exponent_part+'))[FfLl]?)'
+    floating_constant = '(((('+fractional_constant+')'+exponent_part+'?)|([0-9]+'+exponent_part+'))i?[FfLl]?)'
 
     ##
     ## Lexer states
